@@ -9,15 +9,18 @@ SPDX-License-Identifier: GPL-2.0
 
 import datetime
 import weakref
+from types import ModuleType
+from typing import Tuple, Optional, List, Type, Any, TYPE_CHECKING, Union, cast
 
 from twisted.spread import pb
 from twisted.internet.task import deferLater
 from twisted.internet.defer import Deferred, succeed, fail
+from twisted.python.failure import Failure
 from util import Duration
 from log import logDebug, logException, logWarning
 from mi18n import i18nc
 from message import Message
-from common import Internal, Debug, Options, StrMixin
+from common import Internal, Debug, Options, ReprMixin
 from common import isAlive
 from tilesource import TileSource
 from rule import Ruleset
@@ -31,12 +34,23 @@ import intelligence
 import altint
 
 
-class Table(StrMixin):
+if TYPE_CHECKING:
+    from deferredutil import Request
+    from login import Connection
+    from game import Game
+    from tables import TableList
+    from chat import ChatWindow
+    from servertable import ServerTable
+    from wind import Wind
+    from scene import PlayingScene
+    from message import ClientMessage, ServerMessage
+
+class Table(ReprMixin):
 
     """defines things common to both ClientTable and ServerTable"""
 
-    def __init__(self, tableid, ruleset, suspendedAt,
-                 running, playOpen, autoPlay, wantedGame):
+    def __init__(self, tableid:int, ruleset:Union[Ruleset, str], suspendedAt:Optional[str],
+                 running:bool, playOpen:bool, autoPlay:bool, wantedGame:str) ->None:
         self.tableid = tableid
         if isinstance(ruleset, Ruleset):
             self.ruleset = ruleset
@@ -48,7 +62,7 @@ class Table(StrMixin):
         self.autoPlay = autoPlay
         self.wantedGame = wantedGame
 
-    def status(self):
+    def status(self) ->str:
         """a status string"""
         result = ''
         if self.suspendedAt:
@@ -60,19 +74,19 @@ class Table(StrMixin):
             result += ' ' + i18nc('table status', 'Running')
         return result or i18nc('table status', 'New')
 
-    def __str__(self):
-        return 'Table({})'.format(self.tableid)
+    def __str__(self) ->str:
+        return f'Table({self.tableid})'
 
 
 class ClientTable(Table):
 
     """the table as seen by the client"""
-    # pylint: disable=too-many-instance-attributes
     # pylint: disable=too-many-arguments
 
-    def __init__(self, client, tableid, ruleset, gameid, suspendedAt, running,
-                 playOpen, autoPlay, wantedGame, playerNames,
-                 playersOnline, endValues):
+    def __init__(self, client:'Client', tableid:int, ruleset:Ruleset, gameid:int,
+                 suspendedAt:str, running:bool,
+                 playOpen:bool, autoPlay:bool, wantedGame:str, playerNames:List[str],
+                 playersOnline:List[PlayingPlayer], endValues:Tuple[List[int], List[int]]) ->None:
         Table.__init__(
             self,
             tableid,
@@ -92,16 +106,16 @@ class ClientTable(Table):
             if myRuleset == self.ruleset:
                 self.myRuleset = myRuleset
                 break
-        self.chatWindow = None
+        self.chatWindow:Optional['ChatWindow'] = None
 
-    def isOnline(self, player):
+    def isOnline(self, player:Optional[str]) ->bool:  # TODO: why optional?
         """did he join the tabled?"""
         for idx, name in enumerate(self.playerNames):
             if player == name:
-                return self.playersOnline[idx]
+                return bool(self.playersOnline[idx])
         return False
 
-    def __str__(self):
+    def __str__(self) ->str:
         onlineNames = [x for x in self.playerNames if self.isOnline(x)]
         offlineString = ''
         offlineNames = [
@@ -109,9 +123,9 @@ class ClientTable(Table):
             and not x.startswith('Robot')]
         if offlineNames:
             offlineString = ' offline:' + ','.join(offlineNames)
-        return '%d(%s %s%s)' % (self.tableid, self.ruleset.name, ','.join(onlineNames), offlineString)
+        return f"{int(self.tableid)}({self.ruleset.name} {','.join(onlineNames)}{offlineString})"
 
-    def gameExistsLocally(self):
+    def gameExistsLocally(self) ->bool:
         """True if the game exists in the data base of the client"""
         assert self.gameid
         return bool(Query('select 1 from game where id=?', (self.gameid,)).records)
@@ -123,75 +137,76 @@ class Client(pb.Referenceable):
     so we can also use it on the server for robot clients. Compare
     with HumanClient(Client)"""
 
-    def __init__(self, name=None):
+    def __init__(self, name:Optional[str]=None) ->None:
         """name is something like Robot 1 or None for the game server"""
         self.name = name
-        self.game = None
-        self.__connection = None
-        self.tables = []
-        self._table = None
-        self.tableList = None
+        self.game:Optional['PlayingGame'] = None
+        self.__connection:Optional['Connection'] = None
+        self.tables:List[ClientTable] = []
+        self._table:Union[ClientTable, 'ServerTable', None] = None
+        self.tableList:Optional['TableList'] = None
+        self.voiceId:Optional[str]  # only for mypy in servertable.py
 
     @property
-    def table(self):
+    def table(self) ->Union[ClientTable, 'ServerTable', None]:
         """hide weakref"""
         if self._table:
-            return self._table()
+            return self._table()  # type:ignore
         return None
 
     @table.setter
-    def table(self, value):
+    def table(self, value:Union[ClientTable, 'ServerTable', None]) ->None:
         """hide weakref"""
         if value is not None:
-            self._table = weakref.ref(value)
+            self._table = weakref.ref(value)  # type:ignore
 
     @property
-    def connection(self):
+    def connection(self) ->Optional['Connection']:
         """update main window title if needed"""
         return self.__connection
 
     @connection.setter
-    def connection(self, value):
+    def connection(self, value:'Connection') ->None:
         """update main window title if needed"""
         if self.__connection != value:
             self.__connection = value
             if Internal.scene:
                 Internal.scene.mainWindow.updateGUI()
 
-    def _tableById(self, tableid):
+    def _tableById(self, tableid:int) ->Optional[ClientTable]:
         """return table with tableid"""
         for table in self.tables:
             if table.tableid == tableid:
                 return table
         return None
 
-    def logout(self, unusedResult=None):  # pylint: disable=no-self-use
+    def logout(self, unusedResult:Optional[List['Request']]=None) ->Deferred:
         """virtual"""
         return succeed(None)
 
-    def isRobotClient(self):
+    def isRobotClient(self) ->bool:
         """avoid using isinstance because that imports too much for the server"""
         return bool(self.name)
 
     @staticmethod
-    def isHumanClient():
+    def isHumanClient() ->bool:
         """avoid using isinstance because that imports too much for the server"""
         return False
 
-    def isServerClient(self):
+    def isServerClient(self) ->bool:
         """avoid using isinstance because that imports too much for the server"""
         return bool(not self.name)
 
-    def remote_newTables(self, tables):
+    def remote_newTables(self, tables:List[Any]) ->None:
         """update table list"""
         newTables = [ClientTable(self, *x) for x in tables]
         self.tables.extend(newTables)
         if Debug.table:
             _ = ', '.join(str(ClientTable(self, *x)) for x in tables)
-            logDebug('%s got new tables:%s' % (self.name, _))
+            logDebug(f'{self.name} got new tables:{_}')
 
     @staticmethod
-    def remote_serverRulesets(hashes):
+    def remote_serverRulesets(hashes:List[str]) ->List[str]:
         """the server will normally send us hashes of rulesets. If
         a hash is not known by us, tell the server so it will send the
         full ruleset definition instead of the hash. It would be even better if
@@ -200,25 +215,26 @@ class Client(pb.Referenceable):
         more changes to the client code"""
         return [x for x in hashes if not Ruleset.hashIsKnown(x)]
 
-    def tableChanged(self, table):
+    def tableChanged(self, table:Any) ->Tuple[Optional[ClientTable], ClientTable]:
         """update table list"""
-        newTable = ClientTable(self, *table)
+        newTable = ClientTable(self, *table)  # type: ignore
         oldTable = self._tableById(newTable.tableid)
         if oldTable:
             self.tables.remove(oldTable)
-            self.tables.append(newTable)
+            self.tables.append(newTable) # this indent wants a comment
         return oldTable, newTable
 
-    def remote_tableRemoved(self, tableid, message, *args):  # pylint: disable=unused-argument
+    def remote_tableRemoved(self, tableid:int, message:str, *args:Any) ->None:  # pylint: disable=unused-argument
         """update table list"""
         table = self._tableById(tableid)
         if table:
             self.tables.remove(table)
 
-    def reserveGameId(self, gameid):
+    def reserveGameId(self, gameid:int) ->Message:
         """the game server proposes a new game id. We check if it is available
         in our local data base - we want to use the same gameid everywhere"""
         with Internal.db:
+            assert self.connection
             query = Query('insert into game(id,seed) values(?,?)',
                           (gameid, self.connection.url), mayFail=True, failSilent=True)
             if query.rowcount() != 1:
@@ -226,7 +242,7 @@ class Client(pb.Referenceable):
         return Message.OK
 
     @staticmethod
-    def __findAI(modules, aiName):
+    def __findAI(modules:List[ModuleType], aiName:str) ->Optional[Type]:
         """list of all alternative AIs defined in altint.py"""
         for module in modules:
             for key, value in module.__dict__.items():
@@ -234,22 +250,24 @@ class Client(pb.Referenceable):
                     return value
         return None
 
-    def __assignIntelligence(self):
+    def __assignIntelligence(self) ->None:
         """assign intelligence to myself. All players already have default intelligence."""
         if self.isHumanClient():
+            assert self.game
             aiClass = self.__findAI([intelligence, altint], Options.AI)
             if not aiClass:
-                raise Exception('intelligence %s is undefined' % Options.AI)
+                raise ValueError(f'intelligence {Options.AI} is undefined')
             self.game.myself.intelligence = aiClass(self.game.myself)
 
     def readyForGameStart(
-            self, tableid, gameid, wantedGame, playerNames, shouldSave=True, gameClass=None):
+            self, tableid:int, gameid:int, wantedGame:str, playerNames:List[Tuple['Wind', str]],
+            shouldSave:bool=True, gameClass:Optional[Type]=None) ->Deferred:
         """the game server asks us if we are ready. A robot is always ready."""
-        def disagree(about):
+        def disagree(about:str) ->None:
             """do not bother to translate this, it should normally not happen"""
+            assert self.game  # mypy should be able to infer this
             self.game.close()
-            msg = 'The data bases for game %s have different %s' % (
-                self.game.seed, about)
+            msg = f'The data bases for game {self.game.seed} have different {about}'
             logWarning(msg)
             raise pb.Error(msg)
         if not self.table:
@@ -258,23 +276,26 @@ class Client(pb.Referenceable):
         else:
             assert self.isRobotClient()
             # robot client instance: self.table is already set
+        assert self.table
         if gameClass is None:
             gameClass = PlayingGame
         if self.table.suspendedAt:
-            self.game = gameClass.loadFromDB(gameid, self)
+            self.game = cast(PlayingGame, gameClass.loadFromDB(gameid, self))
+            assert self.game, f'cannot load game {gameid}'
             self.game.assignPlayers(playerNames)
+            table = cast(ClientTable, self.table)
             if self.isHumanClient():
-                if self.game.handctr != self.table.endValues[0]:
-                    disagree('numbers for played hands: Server:%s, Client:%s' % (
-                        self.table.endValues[0], self.game.handctr))
+                if self.game.handctr != table.endValues[0]:
+                    disagree(f'numbers for played hands: Server:{table.endValues[0]}, Client:{self.game.handctr}')
                 for player in self.game.players:
-                    if player.balance != self.table.endValues[1][player.wind.char]:
-                        disagree('balances for wind %s: Server:%s, Client:%s' % (
-                            player.wind, self.table.endValues[1][player.wind], player.balance))
+                    if player.balance != table.endValues[1][player.wind.char]:
+                        disagree(f'balances for wind {player.wind}: '
+                                 f'Server:{table.endValues[1][player.wind]}, Client:{player.balance}')
         else:
             self.game = gameClass(playerNames, self.table.ruleset,
                                   gameid=gameid, wantedGame=wantedGame, client=self,
                                   playOpen=self.table.playOpen, autoPlay=self.table.autoPlay)
+            assert self.game, f'cannot initialize game {gameid}'
         self.game.shouldSave = shouldSave
         self.__assignIntelligence()
                                   # intelligence variant is not saved for
@@ -282,14 +303,18 @@ class Client(pb.Referenceable):
         self.game.prepareHand()
         return succeed(Message.OK)
 
-    def readyForHandStart(self, playerNames, rotateWinds):
+    def readyForHandStart(self, playerNames:List[Tuple['Wind', str]],  # pylint:disable=useless-return,useless-suppression
+        rotateWinds:bool) ->Optional[Deferred]:
         """the game server asks us if we are ready. A robot is always ready..."""
-        self.game.assignPlayers(playerNames)
-        if rotateWinds:
-            self.game.rotateWinds()
-        self.game.prepareHand()
+        if self.game:
+            self.game.assignPlayers(playerNames)
+            if rotateWinds:
+                self.game.rotateWinds()
+            self.game.prepareHand()
+        # mypy 1.5.1 wants an explicit return
+        return None
 
-    def __delayAnswer(self, result, delay, delayStep):
+    def __delayAnswer(self, result:List['Request'], delay:float, delayStep:float) ->Any:
         """try again, may we chow now?"""
         if not self.game:
             # game has been aborted meanwhile
@@ -303,55 +328,57 @@ class Client(pb.Referenceable):
             if move.message == Message.NoClaim and move.notifying:
                 noClaimCount += 1
                 if noClaimCount == 2:
-                    if Debug.delayChow:
-                        self.game.debug('everybody said "I am not interested", so {} claims chow now for {}'.format(
-                            self.game.myself.name, self.game.lastDiscard.name()))
+                    if Debug.delayChow and self.game.lastDiscard:
+                        self.game.debug(f'everybody said "I am not interested", so {self.game.myself.name} '
+                                        f'claims chow now for {self.game.lastDiscard.name()}')
                     return result
             elif move.message in (Message.Pung, Message.Kong, Message.MahJongg) and move.notifying:
-                if Debug.delayChow:
-                    self.game.debug('{} said {} so {} suppresses Chow for {}'.format(
-                        move.player, move.message, self.game.myself, self.game.lastDiscard.name()).replace('  ', ' '))
+                if Debug.delayChow and self.game.lastDiscard:
+                    self.game.debug(f'{move.player} said {move.message} so {self.game.myself} '
+                                    f'suppresses Chow for {self.game.lastDiscard.name()}'.replace('  ', ' '))
                 return Message.NoClaim
         if delay < self.game.ruleset.claimTimeout * 0.95:
             # one of those slow humans is still thinking
             return deferLater(Internal.reactor, delayStep, self.__delayAnswer, result, delay, delayStep)
-        if Debug.delayChow:
-            self.game.debug('{} must chow now for {} because timeout is over'.format(
-                self.game.myself.name, self.game.lastDiscard.name()))
+        if Debug.delayChow and self.game.lastDiscard:
+            self.game.debug(f'{self.game.myself.name} must chow now '
+                            f'for {self.game.lastDiscard.name()} because timeout is over')
         return result
 
-    def ask(self, move, answers):
+    def ask(self, move:Move, answers:List['ClientMessage']) ->Deferred:
         """place the robot AI here.
         send answer and one parameter to server"""
         delay = 0.0
         delayStep = 0.1
+        assert self.game
         myself = self.game.myself
-        myself.computeSayable(move, answers)
+        cast(PlayingPlayer, myself).computeSayable(move, answers)
         result = myself.intelligence.selectAnswer(answers)
+        assert result
         if result[0] == Message.Chow:
-            if Debug.delayChow:
-                self.game.debug('{} waits to see if somebody says Pung or Kong before saying chow for {}'.format(
-                    self.game.myself.name, self.game.lastDiscard.name()))
+            if Debug.delayChow and self.game.lastDiscard:
+                self.game.debug(f'{self.game.myself.name} waits to see if somebody '
+                                f'says Pung or Kong before saying chow for {self.game.lastDiscard.name()}')
             return deferLater(Internal.reactor, delayStep, self.__delayAnswer, result, delay, delayStep)
         return succeed(result)
 
-    def thatWasMe(self, player):
+    def thatWasMe(self, player:PlayingPlayer) ->bool:
         """return True if player == myself"""
         if not self.game:
             return False
         return player == self.game.myself
 
     @staticmethod
-    def __jellyMessage(value):
+    def __jellyMessage(value:str) ->str:
         """the Message classes are not pb.copyable, convert them into their names"""
         return Message.OK.name if value is None else Message.jelly(value, value)
 
-    def remote_move(self, playerName, command, *unusedArgs, **kwargs): # pylint: disable=unused-argument
+    def remote_move(self, playerName:str, command:str, *unusedArgs:Any, **kwargs:Any) ->Deferred:
         """the server sends us info or a question and always wants us to answer"""
         if Internal.scene and not isAlive(Internal.scene):
             return fail()
         if self.game:
-            player = self.game.playerByName(playerName)
+            player = cast(PlayingPlayer, self.game.playerByName(playerName))
         elif playerName:
             player = PlayingPlayer(None, playerName)
         else:
@@ -360,26 +387,29 @@ class Client(pb.Referenceable):
         if Debug.traffic:
             if self.isHumanClient():
                 if self.game:
-                    self.game.debug('got Move: %s' % move)
+                    self.game.debug(f'got Move: {move!r}')
                 else:
-                    logDebug('got Move: %s' % move)
+                    logDebug(f'got Move: {move!r}')
         if self.game:
             if move.token:
                 if move.token != self.game.handId.token():
                     logException(
-                        'wrong token: %s, we have %s' %
-                        (move.token, self.game.handId.token()))
-        with Duration('Move %s:' % move):
-            return self.exec_move(move).addCallback(self.__jellyMessage)
+                        f'wrong token: {move.token}, we have {self.game.handId.token()}')
+        with Duration(f'{move!r}:'):
+            try:
+                result = self.exec_move(move).addCallback(self.__jellyMessage)
+            except (Exception, Failure) as exc:  # pylint: disable=broad-exception-caught
+                return fail(exc)
+            return result
 
-    def exec_move(self, move):
+    def exec_move(self, move:Move) ->Deferred:
         """mirror the move of a player as told by the game server"""
         message = move.message
-        if message.needsGame and not self.game:
+        if message.needsGame and not self.game:  # type: ignore
             # server already disconnected, see
             # HumanClient.remote_ServerDisconnects
             return succeed(Message.OK)
-        action = message.notifyAction if move.notifying else message.clientAction
+        action = message.notifyAction if move.notifying else message.clientAction  # type: ignore
         game = self.game
         if game:
             game.moves.append(move)
@@ -423,29 +453,32 @@ class Client(pb.Referenceable):
             return answer
         # return answer only after animation ends. Put answer into
         # the Deferred returned by animate().
-        return animate().addCallback(lambda x: answer)
+        return animate().addCallback(lambda x: answer).addErrback(logException)
 
-    def claimed(self, move):
+    def claimed(self, move:Move) ->Optional[Deferred]:
         """somebody claimed a discarded tile"""
+        assert self.game
         if Internal.scene:
-            calledTileItem = Internal.scene.discardBoard.lastDiscarded
+            calledTileItem = cast('PlayingScene', Internal.scene).discardBoard.claimDiscard()
             calledTile = calledTileItem.tile
-            Internal.scene.discardBoard.lastDiscarded = None
         else:
             calledTileItem = None
+            assert self.game.lastDiscard
             calledTile = self.game.lastDiscard
         self.game.lastDiscard = None
+        assert calledTile
         self.game.discardedTiles[calledTile.exposed] -= 1
-        assert calledTile in move.meld, '%s %s' % (calledTile, move.meld)
-        hadTiles = move.meld[:]
-        hadTiles.remove(calledTile)
+        assert move.meld, f'move has no meld: {move!r}'
+        assert calledTile in move.meld, f'{calledTile} {move.meld}'
+        hadTiles = move.meld.without(calledTile)
+        assert move.player
         if not self.thatWasMe(move.player) and not self.game.playOpen:
             move.player.showConcealedTiles(hadTiles)
         move.player.lastTile = calledTile.exposed
         move.player.lastSource = TileSource.LivingWallDiscard
         move.exposedMeld = move.player.exposeMeld(
-            hadTiles,
-            calledTile=calledTileItem or calledTile)
+            hadTiles, # type:ignore[arg-type]
+            calledTile=calledTileItem or calledTile) # type:ignore[arg-type]
 
         if self.thatWasMe(move.player):
             if move.message != Message.Kong:
@@ -457,28 +490,32 @@ class Client(pb.Referenceable):
             return self.ask(move, [Message.OK])
         return None
 
-    def myAction(self, move):
+    def myAction(self, move:Move) ->Deferred:
         """ask myself what I want to do after picking or claiming a tile"""
         # only when all animations ended, our handboard gets focus. Otherwise
         # we would see a blue focusRect in the handboard even when a tile
         # ist still moving from the discardboard to the handboard.
+        assert move.player
         animateAndDo(move.player.getsFocus)
         possibleAnswers = [Message.Discard, Message.Kong, Message.MahJongg]
         if not move.player.discarded:
             possibleAnswers.append(Message.OriginalCall)
-        return self.ask(move, possibleAnswers)
+        return self.ask(move, list(cast('ClientMessage', x) for x in possibleAnswers))
 
-    def declared(self, move):
+    def declared(self, move:Move) ->None:
         """somebody declared something.
         By declaring we mean exposing a meld, using only tiles from the hand.
         For now we only support Kong: in Classical Chinese it makes no sense
         to declare a Pung."""
         assert move.message == Message.Kong
+        assert self.game
+        assert move.player
         if not self.thatWasMe(move.player) and not self.game.playOpen:
-            move.player.showConcealedTiles(move.source)
-        move.exposedMeld = move.player.exposeMeld(move.source)
+            move.player.showConcealedTiles(move.meld)
+        move.exposedMeld = move.player.exposeMeld(move.meld)
         if not self.thatWasMe(move.player):
             self.ask(move, [Message.OK])
 
-    def __str__(self):
+    def __str__(self) ->str:
+        assert self.name
         return self.name
